@@ -15,69 +15,57 @@ import h3
 from sklearn.cluster import AgglomerativeClustering
 
 
-def count_poi(df,
-              points):
+def count_poi(df, points):
     """
     Counts the number POI in each tile
 
     Parameters
     ----------
-    df : GeoDataFrame
+    df : geopandas.GeoDataFrame
         GeoDataFrame containing the tiles (polygons)
-    points : GeoDataFrame
+    points : geopandas.GeoDataFrame
         GeoDataFrame containing the POI
 
     Returns
     --------
-    final_df : GeoDataFrame
+    final_df : geopandas.GeoDataFrame
         GeoDataFrame containing the tiles and the POI count
     """
-    # TODO: Change such that this works for every tessellation method ->
-    #  groupby by sth like input index or even geometry
 
-    pointsInPolygon = gpd.sjoin(df,
-                                points,
-                                how="left",
-                                predicate='contains')
-    pointsInPolygon['count'] = 1
+    pointsInPolygon = gpd.sjoin(df, points, how="left", predicate="contains")
+    pointsInPolygon["count"] = 1
     pointsInPolygon.reset_index(inplace=True)
 
-    tmp_a = pointsInPolygon.groupby(by='quadkey').count()
-    tmp_a = tmp_a['count'].reset_index()
-    tmp_a = tmp_a.sort_values(by="quadkey",
-                              ascending=True)
+    tmp_a = pointsInPolygon.groupby(by="quadkey").count()
+    tmp_a = tmp_a["count"].reset_index()
+    tmp_a = tmp_a.sort_values(by="quadkey", ascending=True)
 
-    tmp_b = df.reset_index().sort_values(by="quadkey",
-                                         ascending=True)
+    tmp_b = df.reset_index().sort_values(by="quadkey", ascending=True)
 
     final_df = pd.merge(tmp_a, tmp_b, on="quadkey")
-    final_gdf = final_df[['quadkey',
-                          'count',
-                          'geometry',
-                          'children_id']]
-    final_gdf = gpd.GeoDataFrame(final_gdf,
-                                 geometry="geometry")
+    final_gdf = final_df[["quadkey", "count", "geometry", "children_id"]]
+    final_gdf = gpd.GeoDataFrame(final_gdf, geometry="geometry")
 
     return final_gdf
 
 
 ##### squares
 
-def get_squares_polyfill(gdf,
-                         zoom_level):
+
+def get_squares_polyfill(gdf, zoom_level):
     """
     Square tessellation based on the quadKeys concept
 
     Parameters
     ----------
-    gdf : GeoDataFrame
+    gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the tiles (polygons)
     zoom_level : int
         Resolution, which controls the square sizes
 
     Returns
     --------
-    gdf : GeoDataFrame
+    gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the squares
     """
     geom_name = gdf.geometry.name
@@ -86,10 +74,7 @@ def get_squares_polyfill(gdf,
     for idx, rows in gdf.iterrows():
         gdf_geometry = rows[geom_name]
         bbox = gdf_geometry.bounds
-        tiles = mercantile.tiles(bbox[0],
-                                 bbox[1],
-                                 bbox[2],
-                                 bbox[3], zoom_level)
+        tiles = mercantile.tiles(bbox[0], bbox[1], bbox[2], bbox[3], zoom_level)
         temp_rows = []
         for tile in tiles:
             temp_row = rows.copy()
@@ -99,7 +84,9 @@ def get_squares_polyfill(gdf,
                 temp_row["quadkey"] = mercantile.quadkey(tile)
 
                 child_ids = mercantile.children(tile)
-                temp_row['children_id'] = list(mercantile.quadkey(c_tile) for c_tile in child_ids)
+                temp_row["children_id"] = list(
+                    mercantile.quadkey(c_tile) for c_tile in child_ids
+                )
 
                 temp_rows.append(temp_row)
         temp_dfs.append(pd.DataFrame(temp_rows))
@@ -107,62 +94,57 @@ def get_squares_polyfill(gdf,
     df = pd.concat(temp_dfs)
     df = df.reset_index(drop=True)
 
-    gdf = gpd.GeoDataFrame(df,
-                           geometry=geom_name,
-                           crs="epsg:4326")
+    gdf = gpd.GeoDataFrame(df, geometry=geom_name, crs="epsg:4326")
 
     return gdf
 
 
 ##### hexagons
 
-def get_h3_hexagons(gdf,
-                    resolution):
+
+def get_h3_hexagons(gdf, resolution):
     """
     Hexagon tessellation based on the h3 implementation of Uber
 
     Parameters
     ----------
-    gdf : GeoDataFrame
+    gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the tiles (polygons)
     resolution : int
         Resolution, which controls the hexagon sizes
 
     Returns
     --------
-    gdf : GeoDataFrame
+    gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the hexagons
     """
     if type(gdf.geometry.iloc[0]) == Polygon:
-        hexs = h3.polyfill(gdf.geometry[0].__geo_interface__,
-                           resolution,
-                           geo_json_conformant=True)
+        hexs = h3.polyfill(
+            gdf.geometry[0].__geo_interface__, resolution, geo_json_conformant=True
+        )
         polygonise = lambda hex_id: Polygon(
             h3.h3_to_geo_boundary(hex_id, geo_json=True)
         )
-        all_polys = gpd.GeoSeries(list(map(polygonise, hexs)),
-                                  index=hexs,
-                                  crs="EPSG:4326"
-                                  )
+        all_polys = gpd.GeoSeries(
+            list(map(polygonise, hexs)), index=hexs, crs="EPSG:4326"
+        )
 
-        gdf = gpd.GeoDataFrame(geometry=all_polys,
-                               crs='EPSG:4326')
+        gdf = gpd.GeoDataFrame(geometry=all_polys, crs="EPSG:4326")
         return gdf
 
     elif type(gdf.geometry.iloc[0]) == MultiPolygon:
         parts_lst = []
         for idx, row in gdf.explode(index_parts=True).loc[0].iterrows():
-            hexs = h3.polyfill(row.geometry.__geo_interface__,
-                               resolution,
-                               geo_json_conformant=True)
+            hexs = h3.polyfill(
+                row.geometry.__geo_interface__, resolution, geo_json_conformant=True
+            )
 
             polygonise = lambda hex_id: Polygon(
-                h3.h3_to_geo_boundary(hex_id,
-                                      geo_json=True)
+                h3.h3_to_geo_boundary(hex_id, geo_json=True)
+            )
+            all_polys = gpd.GeoSeries(
+                list(map(polygonise, hexs)), index=hexs, crs="EPSG:4326"
             )
-            all_polys = gpd.GeoSeries(list(map(polygonise, hexs)),
-                                      index=hexs,
-                                      crs="EPSG:4326")
 
             gdf = gpd.GeoDataFrame(geometry=all_polys)
 
@@ -173,21 +155,21 @@ def get_h3_hexagons(gdf,
 
 ##### adaptive squares
 
-def get_adaptive_squares(input_gdf,
-                         threshold):
+
+def get_adaptive_squares(input_gdf, threshold):
     """
-    Adaptive tessellation. Subdivides all squares of input tessellation where thresholdi is exceeded.
+    Adaptive tessellation. Subdivides all squares of input tessellation where threshold is exceeded.
 
     Parameters
     ----------
-    input_gdf : GeoDataFrame
+    input_gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the tiles (polygons)
     threshold : int
         threshold, which controls the division of squares
 
     Returns
     --------
-    gdf : GeoDataFrame
+    gdf : geopandas.GeoDataFrame
         GeoDataFrame containing the squares
     """
 
@@ -197,8 +179,7 @@ def get_adaptive_squares(input_gdf,
 
     for idx, row in gdf_exceeded.iterrows():
         children = gdf_exceeded.loc[[idx]]["children_id"].values[0]
-        gdf.drop([idx],
-                 inplace=True)
+        gdf.drop([idx], inplace=True)
 
         for child in children:
             new_row = row.copy()
@@ -207,15 +188,14 @@ def get_adaptive_squares(input_gdf,
             new_row["quadkey"] = child
             new_row["geometry"] = box(*mercantile.bounds(child_tile))
             grand_children = mercantile.children(child_tile)
-            new_row['children_id'] = list(mercantile.quadkey(c_tile) for c_tile in grand_children)
+            new_row["children_id"] = list(
+                mercantile.quadkey(c_tile) for c_tile in grand_children
+            )
 
             tmp_df = pd.DataFrame(new_row).transpose()
-            tmp_gdf = gpd.GeoDataFrame(tmp_df,
-                                       geometry="geometry",
-                                       crs="epsg:4326")
+            tmp_gdf = gpd.GeoDataFrame(tmp_df, geometry="geometry", crs="epsg:4326")
 
-            gdf = pd.concat([gdf, tmp_gdf],
-                            axis=0)
+            gdf = pd.concat([gdf, tmp_gdf], axis=0)
 
     gdf.index = gdf.reset_index(drop=True).index
     return gdf
@@ -223,8 +203,8 @@ def get_adaptive_squares(input_gdf,
 
 ##### voronoi diagram
 
-def voronoi_polygons(sp_voronoi_obj,
-                     diameter):
+
+def voronoi_polygons(sp_voronoi_obj, diameter):
     """
     Voronoi Diagram. Create Voronoi polygons based on scipy Voronoi object
 
@@ -239,13 +219,12 @@ def voronoi_polygons(sp_voronoi_obj,
     Returns
     --------
     gdf : shapely.Polygon
-        Voronoi polygon
+        Voronoi polygons
     """
     centroid = sp_voronoi_obj.points.mean(axis=0)
 
     ridge_direction = defaultdict(list)
-    for (p, q), rv in zip(sp_voronoi_obj.ridge_points,
-                          sp_voronoi_obj.ridge_vertices):
+    for (p, q), rv in zip(sp_voronoi_obj.ridge_points, sp_voronoi_obj.ridge_vertices):
         u, v = sorted(rv)
         if u == -1:
             tangent = sp_voronoi_obj.points[q] - sp_voronoi_obj.points[p]
@@ -269,19 +248,24 @@ def voronoi_polygons(sp_voronoi_obj,
         if previous_vertex_id == next_vertex_id:
             dir_j, dir_k = ridge_direction[i, previous_vertex_id]
         else:
-            dir_j, = ridge_direction[i, previous_vertex_id]
-            dir_k, = ridge_direction[i, next_vertex_id]
+            (dir_j,) = ridge_direction[i, previous_vertex_id]
+            (dir_k,) = ridge_direction[i, next_vertex_id]
 
         length = 2 * diameter / np.linalg.norm(dir_j + dir_k)
 
-        finite_part = sp_voronoi_obj.vertices[region[inf_vertex_id + 1:] + region[:inf_vertex_id]]
-        extra_edge = [sp_voronoi_obj.vertices[previous_vertex_id] + dir_j * length,
-                      sp_voronoi_obj.vertices[next_vertex_id] + dir_k * length]
+        finite_part = sp_voronoi_obj.vertices[
+            region[inf_vertex_id + 1 :] + region[:inf_vertex_id]
+        ]
+        extra_edge = [
+            sp_voronoi_obj.vertices[previous_vertex_id] + dir_j * length,
+            sp_voronoi_obj.vertices[next_vertex_id] + dir_k * length,
+        ]
         yield Polygon(np.concatenate((finite_part, extra_edge)))
 
 
 ##### city blocks
 
+
 def split_linestring(df):
     """
     The linestring, which are split up will have the same osmid
@@ -289,11 +273,13 @@ def split_linestring(df):
 
     Parameters
     ----------
-    df: DataFrame of shapely.LineStrings containing more than two points
+    df: pandas.DataFrame
+        dataframe of shapely.LineStrings containing more than two points
 
     Returns
     --------
-    dataset: DataFrame with shapely.linestring with two points each
+    dataset: pandas.DataFrame
+        dataframe with shapely.linestring with two points each
     """
     linestrings = []
     osmid = []
@@ -305,12 +291,13 @@ def split_linestring(df):
         else:
             for i in range(0, len(row["geometry"].coords) - 1):
                 p1 = Point(row["geometry"].coords[i][0], row["geometry"].coords[i][1])
-                p2 = Point(row["geometry"].coords[i + 1][0], row["geometry"].coords[i + 1][1])
+                p2 = Point(
+                    row["geometry"].coords[i + 1][0], row["geometry"].coords[i + 1][1]
+                )
                 linestrings.append(LineString([p1, p2]))
                 osmid.append(row["osmid"])
 
-    dataset = gpd.GeoDataFrame({'osmid': osmid,
-                                'geometry': linestrings})
+    dataset = gpd.GeoDataFrame({"osmid": osmid, "geometry": linestrings})
     return dataset
 
 
@@ -320,24 +307,27 @@ def explode(gdf):
 
     Parameters
     ----------
-    gdf: which can have multi-part geometries that will be exploded
+    gdf:  geopandas.GeoDataFrame
+        GeoDataFrame that can have multi-part geometries that will be exploded
 
     Returns
     --------
-    gdf_out: GeoDataFrame with single geometries
+    gdf_out: geopandas.GeoDataFrame
+        GeoDataFrame with single geometries
     """
     gs = gdf.explode(index_parts=True)
     gdf2 = gs.reset_index().rename(columns={0: "geometry"})
-    gdf_out = gdf2.merge(gdf.drop("geometry", axis=1),
-                         left_on="level_0",
-                         right_index=True, )
+    gdf_out = gdf2.merge(
+        gdf.drop("geometry", axis=1),
+        left_on="level_0",
+        right_index=True,
+    )
     gdf_out = gdf_out.set_index(["level_0", "level_1"]).set_geometry("geometry")
     gdf_out.crs = gdf.crs
     return gdf_out
 
 
-def get_hierarchical_clustering_parameter(coordinates,
-                                          threshold):
+def get_hierarchical_clustering_parameter(coordinates, threshold):
     """
     This function returns the distance_threshold that is used in
     the hierarchical clustering algorithm. Therefore, different
@@ -352,19 +342,24 @@ def get_hierarchical_clustering_parameter(coordinates,
 
     Parameters
     ----------
-    coordinates: numpy column stock of coordinates of Data
-    threshold: Number of Polygons that should not be exceeded
+    coordinates: numpy.ndarray
+        stock of coordinates of Data
+    threshold: int
+        positive number indicating the number of Polygons that should not be exceeded
 
     Returns
     --------
-    th: int, defines the distance_threshold for hierarchical clustering
+    th: int
+        defines the distance_threshold for hierarchical clustering
     """
     dist_threshold = [i * 100 for i in range(2, 13)]
     for th in dist_threshold:
-        model = AgglomerativeClustering(n_clusters=None,
-                                        distance_threshold=th,
-                                        affinity="euclidean",
-                                        compute_full_tree=True)
+        model = AgglomerativeClustering(
+            n_clusters=None,
+            distance_threshold=th,
+            affinity="euclidean",
+            compute_full_tree=True,
+        )
         model.fit(coordinates)
         labels = model.labels_
         nb_clusters = len(set(labels)) - (1 if -1 in labels else 0)
@@ -375,23 +370,24 @@ def get_hierarchical_clustering_parameter(coordinates,
 
 def create_blocks(road_network):
     """
-    This function uses the shapely function polygonize to create
-    blocks by using road data.
+    This function uses polygonize to create blocks by using road data.
 
     Parameters
     ----------
-    road_network: GeoDataFrame from RoadData class containing street segments
+    road_network: geopandas.GeoDataFrame
+        GeoDataFrame (coming from RoadData class) containing street segments
 
     Returns
     --------
-    blocks: GeoDataFrame with polygons, that were created by using the road data
+    blocks: geopandas.GeoDataFrame
+        GeoDataFrame with polygons, that were created by using the road data
     """
-    if hasattr(road_network, 'geometry'):
-        block_faces = list(polygonize(road_network['geometry']))
+    if hasattr(road_network, "geometry"):
+        block_faces = list(polygonize(road_network["geometry"]))
         blocks = gpd.GeoDataFrame(geometry=block_faces).set_crs("EPSG:4326")
         return blocks
     else:
-        raise AttributeError('road network data need geometry attribute!')
+        raise AttributeError("road network data need geometry attribute!")
 
 
 def get_rest_polygon(blocks, area):
@@ -399,20 +395,25 @@ def get_rest_polygon(blocks, area):
     #  groups with ocean and multiple polygons because now the convex
     #  hull is used. That results in ocean parts becoming city blocks
     """
-    This function creates the rest polygons that can occur by creating city blocks. Dead ends or vegetation at the
-    boundary of the area will not be defined as blocks because there are no road to define a block. Since the
-    tessellation should cover the whole area those "rest polygons" will be  created by subtracting all blocks from the
-    area to fill these gaps. The rest_polygons are most likely a Multi-Polygon which is exploded into many polygons.
+    This function creates the rest polygons that can occur by creating
+    city blocks. Dead ends or vegetation at the boundary of the area will
+    not be defined as blocks because there are no road to define a block.
+    Since the tessellation should cover the whole area those "rest polygons"
+    will be created by subtracting all blocks from the area to fill these gaps.
+    The rest_polygons are most likely a Multi-Polygon which is exploded into many polygons.
     Using this, the city blocks method tessellates the whole area without gaps.
 
     Parameters
     ----------
-    blocks: GeoDataFrame with all blocks
-    area: GeoDataFrame with boundary polygon
+    blocks: geopandas.GeoDataFrame
+        GeoDataFrame containing city blocks
+    area: geopandas.GeoDataFrame
+        boundary polygon
 
     Returns
     --------
-    rest_polygons: GeoDataFrame of the rest polygons
+    rest_polygons: geopandas.GeoDataFrame
+        GeoDataFrame containing rest polygons
     """
 
     if hasattr(blocks, "geometry") and hasattr(area, "geometry"):
@@ -420,9 +421,7 @@ def get_rest_polygon(blocks, area):
         blocks["geometry"] = blocks["geometry"].apply(lambda x: make_valid(x))
 
         merged_polygons = gpd.GeoSeries(unary_union(blocks["geometry"].values))
-        merged_polygons.set_crs("EPSG:4326",
-                                allow_override=True,
-                                inplace=True)
+        merged_polygons.set_crs("EPSG:4326", allow_override=True, inplace=True)
 
         rest = area.difference(merged_polygons)
         rest = gpd.GeoDataFrame(rest)
@@ -430,12 +429,12 @@ def get_rest_polygon(blocks, area):
 
         rest_polygons = explode(rest)
         rest_polygons.reset_index(inplace=True)
-        rest_polygons.drop(columns=["level_0"],
-                           inplace=True)
-        rest_polygons.rename(columns={"level_1": "osm_id"},
-                             inplace=True)
+        rest_polygons.drop(columns=["level_0"], inplace=True)
+        rest_polygons.rename(columns={"level_1": "osm_id"}, inplace=True)
 
         return rest_polygons
 
     else:
-        raise ValueError("Initial defined city blocks and the area need a geometry attribute.")
+        raise ValueError(
+            "Initial defined city blocks and the area need a geometry attribute."
+        )
